# siujeless

A serverless version for [siuje](https://www.npmjs.com/package/siuje), the middleware
using in siuje can be reused on siujeless without modification.

# How to use

Generally you can make a fech event listener, for example, in Bun, using code like:

```javascript
import siuje from "siujeless"

// sample for middleware, copy from siuje
const set_header = headers => {
  return (req, ctrl) => next => {
    ctrl( headers );
    return next(req, ctrl);
  }
}

Bun.serve({
  port: 10001,
  fetch: siuje([
    set_header({"server": "siujeless"}),
    async (req, ctrl) => {
      ctrl(200);
      return "hello,world";
    }
  ]),
});
```

So with siujeless, your code in siuje can move to serverless easily.

# Why no router

Siujeless is developed for using in cloudflare page functions. It used 
folder router. If you need a router for some runtime environment like 
service worker, [find-my-way](https://www.npmjs.com/package/find-my-way) is suggested.

```javascript
import siuje from "siujeless";
import fmw from "find-my-way";

const set_header = headers => {
  return (req, ctrl) => next => {
    ctrl( headers );
    return next(req, ctrl);
  }
}

const router = fmw();
router.on("GET", "/hello/:name", siuje(
  set_header({server: "siujeless"}),
  async (req, ctrl) => `hello, ${req.params.name}!!`
));

Bun.serve({
  port: 10001,
  fetch: async req => {
    const url = new URL(req.url);
    const result = router.find(req.method, url.pathname);
    if (result) {
      req.params = result.params;
      return await result.handler( req );
    }
    else return new Response("not found", {status: 404});
  }
});
```
